/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Properties;

/**
 *
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a><br />
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a><br />
 * <br />
 * <br />
 * This class is for making WikiInsidePlugins.<br />
 * <br />
 * The only difference is that they are processed in HTMLTranslatorParts plugin and return
 * String to insert inside showed page.<br />
 * <br />
 * WikiSession attributes are the same as in {@link WikiPlugin} <br /> 
 * <br />
 * To add definition of WikiInsidePlugin you need to add
 * coresponding part to <i>wiki-common/src/etc/WikiInsidePlugins.xml:</i>
 * <pre>
 * &lt;entry key="<i>unique plugin name - this will be used in page source</i>"&gt;<i>plugin class name</i>&lt;/entry&gt;
 * </pre>
 * @see WikiPlugin 
 */
public abstract class WikiInsidePlugin {
	 
	protected WikiEngine wikiEngine;
	
	private Properties pluginProps;
	
	private String name;
	
	final static String  propFileName = "wikiPlugins.properties";

    /**
     * Processes the given page, and attributes and return String to show in the apropriate place on the page.
     * 
     * @param wikiPage Page where from the plugin was executed.
     * @param wikiSession Session for passing attibutes etc.
     * @param attributes Attributes given to plugin.
     * @return String to show.
     */
    public abstract String process(final WikiPage wikiPage, WikiSession wikiSession, HashMap<String, String> attributes);
    
    /**
     * It's run after initiating plugin.
     */
    public abstract void init();
    
    /**
     * Sets wikiEngine.
     * 
     * @param wikiEngine WikiEngine to set.
     */
    public void setWikiEngine(WikiEngine wikiEngine) {
    	this.wikiEngine = wikiEngine;
    	
    	// we can treat it as "init" function - it's executed after creating instance of plugin
    	// load props
    	
    	pluginProps = new Properties();
    	
    	try {
    		
    		InputStream propsStream = FileDataSource.class.getResourceAsStream(propFileName);
    		
    		pluginProps.load(propsStream);
    	}
    	catch (Exception e) {
    		System.err.println("[WikiPlugin]: Cannot load WikiInsidePlugin property file: "+e);
    	}
    }
    
    /**
     * Gets property from wikiPlugins.properties
     * 
     * @param propertyName Name of the property to get
     * @return Apripriate property.
     */
    protected String getProperty(String propertyName) {
    	return pluginProps.getProperty(propertyName);
    }

    /**
	 * Returns name of that plugin.
	 * 
	 * @return name of current plugin.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets name of current plugin.
	 * 
	 * @param name Name of the plugin.
	 */
	public void setName(String name) {
		this.name = name;
	}
    

 }
