/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.InputStream;
import java.util.Properties;

import org.jboss.logging.Logger;

/**
 *
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a><br />
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a><br />
 * <br />
 * <br />
 * This class is used for making own plugins, that allow page processing.<br />
 * <br />
 * From inside WikiPlugin you can acess WikiEngine, WikiPage you're processing and some session attributes passed to the plugin in WikiSession.<br />
 * <br />
 * They are:<br />
 * <br />
 * <table>
 * <tr><th>Attribute</th><th>Type</th><th>What is it?</th></tr>
 * <tr><td>actionURL</td><td>String</td><td>Action url to create links</td></tr>
 * <tr><td>hostURL</td><td>String</td><td>Url of the host</td></tr>
 * <tr><td>credentials</td><td>org.jboss.wiki.Credentials</td><td>Credentials of browsing user</td></tr>
 * </table>
 * <br />
 * Also if you add to actionURL parameter val1 val2 ... valn they will be all added to WikiSesion and passed to plugins.<br />
 * <br />
 * To add a plugin to a WikiType you have to add coresponding xml part to 
 * <i>wiki-common/src/etc/WikiTypes.xml</i>:<br />
 * <pre>
 * 	&lt;plugin&gt;
 * 		&lt;name&gt;<i>plugin name</i>&lt;/name&gt;
 * 		&lt;class&gt;<i>plugin class</i>&lt;name&gt;
 * 	&lt;/plugin&gt;
 * </pre>
 * @see WikiInsidePlugin
 */
public abstract class WikiPlugin {
	 
	protected WikiEngine wikiEngine;
	
	private Properties pluginProps;
	
	private String name;
	
	final static String  propFileName = "wikiPlugins.properties";

	protected Logger log = Logger.getLogger(this.getClass());
	
    /**Processes given WikiPage and returns processed one.
     * 
     * @param wikiPage WikiPage to process
     * @param wikiSession Session for passing attributes and getting some stuff from it.
     * @return Processed WikiPage
     */
    public abstract WikiPage process(final WikiPage wikiPage, WikiSession wikiSession);
    
    /**
     * It's run after initiating plugin.
     */
    public abstract void init();
    
    /**
     * Sets WikiEngine. If you're not sure why would you like to override it, leave that one.
     * 
     * @param wikiEngine
     */
    public void setWikiEngine(WikiEngine wikiEngine) {
    	this.wikiEngine = wikiEngine;
    	
    	// we can treat it as "init" function - it's executed after creating instance of plugin
    	// load props
    	
    	pluginProps = new Properties();
    	
    	try {
    		
    		InputStream propsStream = FileDataSource.class.getResourceAsStream(propFileName);
    		
    		pluginProps.load(propsStream);
    	}
    	catch (Exception e) {
    		log.error("Cannot load WikiPlugin property file: "+e);
    	}
    }
    
    /**
     * Gets property from wikiPlugins.properties
     * 
     * @param propertyName Name of the property to get
     * @return Apripriate property.
     */
    protected String getProperty(String propertyName) {
    	return pluginProps.getProperty(propertyName);
    }

	/**
	 * Returns name of that plugin.
	 * 
	 * @return name of current plugin.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets name of current plugin.
	 * 
	 * @param name Name of the plugin.
	 */
	public void setName(String name) {
		this.name = name;
	}
    
 }
