/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.util.Stack;

import org.jboss.logging.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * The WikiTypeHandler for parsing WikiType xml file.
 * <p>
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szimanski@jboss.com">Tomasz Szimanski</a>
 * @version $Id$
 */

public class WikiTypeHandler extends DefaultHandler {

	private Stack<String> elements = new Stack<String>();

	private boolean isInsidePlugin = false;

	private WikiEngine wikiEngine;

	private WikiType wikiType;

	private String wikiTypeName;

	private String pluginName = "", pluginClass = "";
	
	private Logger log = Logger.getLogger(WikiTypeHandler.class);

	@Override
	public void endDocument() throws SAXException {
		super.endDocument();
		
		log.info("WikiType configuration file parsed.");
	}

	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		super.endElement(uri, localName, qName);

		if (qName.equals("plugin")) {
			isInsidePlugin = false;
		} else if ((qName.equals("wikiType")) && (wikiType != null)) {
			// add WikiType
			
			wikiEngine.addWikiType(wikiType.getName(), wikiType);
			wikiType = null;
			wikiTypeName = null;
		}

		// log.info("End element: "+uri+" "+localName+" "+qName);
	}

	@Override
	public void error(SAXParseException e) throws SAXException {
		super.error(e);
		log.error("ERROR", e);
	}

	@Override
	public void fatalError(SAXParseException e) throws SAXException {
		super.fatalError(e);
		log.error("FATAL ERROR" , e);
	}

	@Override
	public void startDocument() throws SAXException {
		super.startDocument();
		log.info("Parsing the WikiType configuration file...");
	}

	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		super.startElement(uri, localName, qName, attributes);
		/*
		 * log.info("Element: uri: "+uri+" lname: "+localName+" qname:
		 * "+qName); for (int i = 0; i < attributes.getLength(); i++) {
		 * log.info("Attr "+i+": "+attributes.getQName(i)+" =
		 * "+attributes.getIndex(attributes.getQName(i))); }
		 */
		if (qName.equals("plugin")) {
			isInsidePlugin = true;
		} else if (qName.equals("wikiType")) {
			// start building new WikiType

			//wikiType = new WikiType();
		}

		elements.push(qName);
	}

	@Override
	public void warning(SAXParseException e) throws SAXException {
		super.warning(e);
		log.warn(e);
	}

	public WikiTypeHandler(WikiEngine wikiEngine) {
		this.wikiEngine = wikiEngine;
	}

	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		super.characters(ch, start, length);
		String element;

		String s = (new String(ch)).substring(start, start + length);

		if (((int) s.charAt(0) != 9)
				&& ((int) s.charAt(0) != 10)
				&& (!elements.isEmpty())) {

			element = elements.pop();
			
			if ((element.equals("class")) && (!isInsidePlugin)) {
				// class for wikiType
				
				try {
					wikiType = (WikiType)Class.forName(s).newInstance();
					wikiType.setWikiEngine(wikiEngine);
				}
				catch(Exception e) {
					log.error("Cannot load WikiType class: ",e);
				}
				
				if (wikiTypeName != null) {
					wikiType.setName(wikiTypeName);
					
					wikiTypeName = null;
				}
				
			} else if ((element.equals("name")) || (element.equals("class"))) {
				if (isInsidePlugin) {
					// set name and class - if we have them both - add plugin to
					// wikitype
					
					if (element.equals("class")) {
						pluginClass = s;
					} else {
						pluginName = s;
					}


					if ((pluginName.length() > 0) && (pluginClass.length() > 0)) {
						// we have em both - add a plugin
						
						wikiType.addPlugin(pluginName, pluginClass);
						
						pluginClass = pluginName = "";
					}
					
					
				} else {
					// here only if found name of WikiType (see prev. if clause)
					if (wikiType != null) {
						wikiType.setName(s);
					}
					else {
						wikiTypeName = s;
					}
				}
			}
		}
	}
}
